@@ -14,10 +14,32 @@ from jax._src.api import (_check_callable, _check_input_dtype_jacfwd, _check_inp
                           _jvp, _vjp, _std_basis, _jacfwd_unravel, _jacrev_unravel, lu, argnums_partial, tree_map, tree_structure, tree_transpose, partial, Callable, Sequence, Union, vmap)
 
 
-def jvp_vmap(func, primals, tangents):
-    """vmap over jvp. Currently only for functions with two arguments."""
-    pushfwd = functools.partial(jax.jvp, func, primals)
-    return jax.vmap(pushfwd, in_axes=([1, 1],), out_axes=(None, 1))(tangents)
+def jvp_vmap(fun: Callable, argnums: Union[int, Sequence[int]] = 0):
+    """Vectorized (forward-mode) Jacobian-vector product of ``fun``. This is by large adopted from the implementation of jacfwd in jax._src.api.
+
+    Args:
+      fun: Function whose value and Jacobian is to be computed.
+      argnums: Optional, integer or sequence of integers. Specifies which
+        positional argument(s) to differentiate with respect to (default ``0``).
+
+    Returns:
+      A function with the same arguments as ``fun``, that evaluates the value and Jacobian of
+      ``fun`` using forward-mode automatic differentiation.
+    """
+    _check_callable(fun)
+    argnums = _ensure_index(argnums)
+
+    def jacfun(args, tangents, **kwargs):
+
+        f = lu.wrap_init(fun, kwargs)
+        f_partial, dyn_args = argnums_partial(
+            f, argnums, args, require_static_args_hashable=False)
+        pushfwd = partial(_jvp, f_partial, dyn_args)
+        y, jac = vmap(pushfwd, out_axes=(None, -1), in_axes=-1)(tangents)
+
+        return y, jac
+
+    return jacfun
 
 
 def jacfwd_and_val(fun: Callable, argnums: Union[int, Sequence[int]] = 0,
@@ -179,8 +201,8 @@ def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=No
 
         if func_returns_jac:
             fval, jacval = func(xi)
-            if isinstance(jacval, ssp._arrays.lil_array):
-                jacval = jacval.tocsr()
+            if sparse and not isinstance(jacval, ssp._arrays.csr_array):
+                jacval = ssp.csr_array(jacval)
         else:
             fval, jacval = func(xi), jac(xi)
 
